""" event handler. """

import logging
import obj
import obj.loader
import obj.tasks
import obj.utils
import queue
import sys
import time
import threading

class Handler(obj.loader.Loader):

    def __init__(self):
        super().__init__()
        self._curevent = None
        self._queue = queue.Queue()
        self._ready = threading.Event()
        self._stopped = False
        self._threaded = True
        self._thrs = []
        self.state = obj.Object()
        self.verbose = False

    def dispatch(self, event):
        event.parse(event.txt)
        event._func = self.get_cmd(event)
        event.verbose = self.verbose
        if event._func:
            logging.warn("dispatch %s" % event.txt)
            try:
                event._func(event)
                event.show()
            except Exception as ex:
                logging.error(obj.get_exception())
        event.ready()
        return event

    def get_cmd(self, event):
        return self.get_handler(event.cmd)

    def get_event(self):
        return self._queue.get()

    def loop(self):
        while not self._stopped:
            try:
                event = self.get_event()
            except EOFError:
                break
            if not event:
                continue
            thr = obj.launch(self.dispatch, event)
            event._thrs.append(thr)

    def put(self, event):
        self._queue.put_nowait(event)

    def ready(self):
        self._ready.set()

    def start(self):
        obj.launch(self.loop)

    def stop(self):
        self._stopped = True
        self._queue.put(None)

    def wait(self):
        return self._ready.wait()

class Worker(Handler):

    def loop(self):
        while not self._stopped:
            try:
                event = self._queue.get()
            except EOFError:
                break
            if not event:
                continue
            thr = obj.launch(self.dispatch, event)
            event._thrs.append(thr)
